#!/bin/bash
#
# Create retroarch config file per game and write resolution index
#
# To re-create GX_Resolutions.txt(in case it's changed in Retroarch): DO_GX_RES=1
# For Mame2003_Resolutions.txt : DO_MAME_RES=1 
# GX_RESOLUTIONS_DEFAULT starts at 0. GX_RESOLUTIONS_608_224 = 9
#                       
# mame2003.xml comes from libretro database.
# FBA_WORKING_NOCLONES_NONEOGEO.csv is from the included .xlsx 

DATA_DIR="DATAS/"          
DATA_FILE=""
FBA=0
FBANEO=0
MAME=0
DO_MAME_RES=0
DO_GX_RES=0
SUFFIX=""
DONE=0

rm ${DATA_DIR}ListZip.txt



echo ""
echo "This script generates a .cfg file for each game with the corresponding resolution."
echo "Select a system and press Enter:"
echo ""
echo "1. FBA"
echo "2. FBA NEOGEO"
echo "3. MAME"
echo "q. EXIT"
echo ""

while [ $DONE != 1 ]
do
    reply=""
    read -p "" choice
    case $choice in 
        1) 
            DATA_FILE="${DATA_DIR}FBA_WORKING_NOCLONES_NONEOGEO.csv"
            FBA=1
            DONE=1
            ;;
        2) 
            grep "DRV"   ${DATA_DIR}driverlist_neogeo.h | grep -v "#"  | sed 's/^.*BurnDrv//g' | sed 's/;//g' | cut -d " " -f1 | tr '[:upper:]' '[:lower:]' | sort -u  > ${DATA_DIR}ListZip.txt
            DATA_FILE="${DATA_DIR}Mame2003_Resolutions.txt"
            FBANEO=1
            DONE=1
            ;;
        3) 
            DATA_FILE="${DATA_DIR}Mame2003_Resolutions.txt"
            MAME=1
            DONE=1
            ;;
        q|Q) 
            exit
            ;;
        *) 
            echo "Try again..."
            ;;
    esac
done

if [[ "$FBA" -eq 1 ]]; then
    SUFFIX="FBA"
elif [[ "$FBANEO" -eq 1 ]]; then
    SUFFIX="FBANEO"
elif [[ "$MAME" -eq 1 ]]; then
    SUFFIX="MAME"
fi

[ -f Missing_$SUFFIX.txt ] && rm Missing_$SUFFIX.txt

if [ ! -d out$SUFFIX ];then
    mkdir out$SUFFIX 
    mkdir out$SUFFIX/CRC
    mkdir out$SUFFIX/LCD
fi

#---------------------------------------------------------------------------------
# Create Mame resolutions file based on libretro xml
#---------------------------------------------------------------------------------
if [ "$DO_MAME_RES" -eq 1 ];then
    rm $DATA_FILE
    grep "<game name" ${DATA_DIR}mame2003.xml > tempnames
    grep "<video " ${DATA_DIR}mame2003.xml > tempvideo
    paste -d"|" tempnames tempvideo  > temp

    echo "Creating Mame2003_Resolutions.txt..."

    while read line
    do
        NAME=`echo $line | cut -d "\"" -f2`
        CLONE=`echo $line | grep "cloneof=" | cut -d "\"" -f4`
        ORIENT=`echo $line | grep "orientation="  | sed -n -e 's/^.*orientation=//p' | cut -d "\"" -f2`
        WIDTH=`echo $line | grep "width="  | sed -n -e 's/^.*width=//p' | cut -d "\"" -f2`
        HEIGHT=`echo $line | grep "height="  | sed -n -e 's/^.*height=//p' | cut -d "\"" -f2`

        # Swap Width & height values for vertical games.
        if [[ "$ORIENT" == "vertical" ]]; then
            TMP=$WIDTH
            WIDTH=$HEIGHT
            HEIGHT=$TMP
        fi

        echo "$NAME|$CLONE|$ORIENT|$WIDTH|$HEIGHT" >> $DATA_FILE
    done < temp

    rm temp; rm tempvideo; rm tempnames;
fi

#---------------------------------------------------------------------------------
# Create GX resolutions file
#---------------------------------------------------------------------------------
if [ "$DO_GX_RES" -eq 1 ];then
    grep GX_RESOLUTIONS_ ${DATA_DIR}gx_gfx.c | grep  -v GX_RESOLUTIONS_LAST | cut -d "," -f1 | sed "s/[[:blank:]]//g" | cut -d "=" -f1 > ${DATA_DIR}GX_Resolutions.txt
    echo "Creating GX_Resolutions.txt..."
fi

# Associative array : GX resolution name = internal Retroarch index 
declare -A GX_RESOLUTIONS
i=0

while read line; do
    GX_RESOLUTIONS[$line]=$(( $i ))
    (( i++ ))
done < ${DATA_DIR}GX_Resolutions.txt


#---------------------------------------------------------------------------------
# Create filenames list
#---------------------------------------------------------------------------------
if [ "$MAME" -eq 1 ];then
    for index in A B C D E F G ; do
        cat ${DATA_DIR}core${index}.txt >> ${DATA_DIR}ListZip.txt
    done
elif [ "$FBA" -eq 1 ];then
    cut -d "|" -f1 $DATA_FILE >> ${DATA_DIR}ListZip.txt
fi


#---------------------------------------------------------------------------------
# Search for game's resolution and create .cfg file
#---------------------------------------------------------------------------------
echo "Creating $SUFFIX .cfg files..."

while read line
do
    CFGPATH_CRC=""
    CFGPATH_LCD=""

    # Specific FBA/MAME output paths/names.
    if [ "$FBA" -eq 1 ]; then
        NAME=`echo $line`
        CFGNAME=`echo $line | sed "s/.zip//g"`
        PLATFORM=`grep "^${NAME}|" $DATA_FILE  | cut -d "|" -f12`

        # Seperate per core config.
        if [[ "$PLATFORM" == "CAPCOM CPS 1" ]]; then
            PLATFORM="CPS1"
        elif [[ "$PLATFORM" == "CAPCOM CPS 2" ]]; then
            PLATFORM="CPS2"
        else
            PLATFORM="FBA"
        fi

        if [[ -z "${PLATFORM}" ]]; then
            continue
        fi

        if [ ! -d "out$SUFFIX/CRC/${PLATFORM}" ]; then
            mkdir "out$SUFFIX/CRC/${PLATFORM}"
            mkdir "out$SUFFIX/LCD/${PLATFORM}"
        fi

        CFGPATH_CRC="out$SUFFIX/CRC/${PLATFORM}/${CFGNAME}.cfg"
        CFGPATH_LCD="out$SUFFIX/LCD/${PLATFORM}/${CFGNAME}.cfg"
    else
        NAME=`echo $line | sed "s/.zip//g"`
        CFGNAME=`echo $line | sed "s/.zip//g"`
        CFGPATH_CRC="out$SUFFIX/CRC/$CFGNAME.cfg"
        CFGPATH_LCD="out$SUFFIX/LCD/$CFGNAME.cfg"
    fi

    # Search game in data file
	if grep -q "^${NAME}|" $DATA_FILE; then
		ORIENT=`grep "^${NAME}|" $DATA_FILE | cut -d "|" -f3`
		WIDTH=`grep "^${NAME}|" $DATA_FILE  | cut -d "|" -f4`
		HEIGHT=`grep "^${NAME}|" $DATA_FILE  | cut -d "|" -f5`
		#echo "$NAME;$WIDTH;$HEIGHT;$ORIENT"

        # Mininal retroarch width is 336
        if [ "$WIDTH" -le 320 ];then
            WIDTH=$(($WIDTH * 2))
        fi

        GX_LOWRES="GX_RESOLUTIONS_${WIDTH}_${HEIGHT}"

        # 240p only for CRC. Multiply by 2 for LCD display except 512_384(same for both).
        if [ "$HEIGHT" -le 240 ];then
            HEIGHT=$(($HEIGHT * 2))
        fi

        GX_HIGHRES="GX_RESOLUTIONS_${WIDTH}_${HEIGHT}"

        # Copy retroarch GX index to config if this lowest res exist.
		if [[ -n "${GX_RESOLUTIONS[$GX_LOWRES]}" ]]; then
            #echo "$NAME||$GX_LOWRES||${GX_RESOLUTIONS[$GX_LOWRES]}||${GX_RESOLUTIONS[$GX_HIGHRES]}"
			touch "${CFGPATH_CRC}"
			echo "current_resolution_id = \"${GX_RESOLUTIONS[$GX_LOWRES]}\"" > "${CFGPATH_CRC}"
			touch "${CFGPATH_LCD}"
			echo "current_resolution_id = \"${GX_RESOLUTIONS[$GX_HIGHRES]}\"" > "${CFGPATH_LCD}"
		else
			echo "$NAME" >> Missing_$SUFFIX.txt
		fi

	fi		
done < ${DATA_DIR}ListZip.txt


exit


